@@ -21,19 +21,20 @@
 
 # Imports
 from bip_utils.bech32 import AtomBech32Encoder
-from bip_utils.utils  import CryptoUtils, KeyUtils
+from bip_utils.utils import CryptoUtils, KeyUtils
 
 
 class AtomAddr:
     """ Atom address class. It allows the Atom address generation. """
 
     @staticmethod
-    def ToAddress(pub_key_bytes, hrp):
-        """ Get address in P2WPKH format.
+    def ToAddress(pub_key_bytes: bytes,
+                  hrp: str) -> str:
+        """ Get address in Atom format.
 
         Args:
             pub_key_bytes (bytes): Public key bytes
-            hrp (bytes)          : Net address version, default is Bitcoin main network
+            hrp (str)            : HRP
 
         Returns:
             str: Address string
