@@ -1,7 +1,7 @@
 # BIP39
 from bip_utils.bip.bip39_ex import Bip39InvalidFileError, Bip39ChecksumError
 from bip_utils.bip.bip39 import (
-    Bip39WordsNum, Bip39EntropyBitLen,
+    Bip39WordsNum, Bip39EntropyBitLen, Bip39Languages,
     Bip39EntropyGenerator, Bip39MnemonicGenerator, Bip39MnemonicValidator, Bip39SeedGenerator
 )
 # BIP32
