@@ -51,6 +51,7 @@ class Bip44Const:
             Bip44Coins.BAND_PROTOCOL,
             Bip44Coins.KAVA,
             Bip44Coins.IRIS_NET,
+            Bip44Coins.TERRA,
             Bip44Coins.BINANCE_CHAIN,
             Bip44Coins.BINANCE_SMART_CHAIN,
             Bip44Coins.AVAX_C_CHAIN,
@@ -83,6 +84,7 @@ class Bip44Const:
             Bip44Coins.BAND_PROTOCOL: Bip44BandProtocol,
             Bip44Coins.KAVA: Bip44Kava,
             Bip44Coins.IRIS_NET: Bip44IrisNet,
+            Bip44Coins.TERRA: Bip44Terra,
             Bip44Coins.BINANCE_CHAIN: Bip44BinanceChain,
             Bip44Coins.BINANCE_SMART_CHAIN: Bip44BinanceSmartChain,
             Bip44Coins.AVAX_C_CHAIN: Bip44AvaxCChain,
