@@ -53,6 +53,7 @@ class Bip44Coins(Enum):
     BAND_PROTOCOL = auto(),
     KAVA = auto(),
     IRIS_NET = auto(),
+    TERRA = auto(),
     BINANCE_CHAIN = auto(),
     BINANCE_SMART_CHAIN = auto(),
     AVAX_C_CHAIN = auto(),
@@ -112,6 +113,7 @@ class Bip44BaseConst:
             Bip44Coins.BITCOIN_SV: 236,
             Bip44Coins.BAND_PROTOCOL: 494,
             Bip44Coins.KAVA: 494,
+            Bip44Coins.TERRA: 330,
             Bip44Coins.NINE_CHRONICLES_GOLD: 567,
             Bip44Coins.BINANCE_CHAIN: 714,
             Bip44Coins.VECHAIN: 818,
@@ -236,8 +238,7 @@ class Bip44Base(ABC):
         # If the Bip32 is public-only, the depth shall start from the account level because hardened derivation is
         # used below it, which is not possible with public keys
         if bip32_obj.IsPublicOnly():
-            if bip32_obj.Depth() < Bip44Levels.ACCOUNT or \
-                    bip32_obj.Depth() > Bip44Levels.ADDRESS_INDEX:
+            if (bip32_obj.Depth() < Bip44Levels.ACCOUNT or bip32_obj.Depth() > Bip44Levels.ADDRESS_INDEX):
                 raise Bip44DepthError(
                     "Depth of the public-only Bip32 object (%d) is below account level or beyond address index level" %
                     bip32_obj.Depth())
